@@ -39,7 +39,6 @@
 
 #include <libinput.h>
 #include <libevdev/libevdev.h>
-#include "config.h"
 #include "shared.h"
 #include "consolation.h"
 
@@ -258,11 +257,23 @@ usage(void)
          "--disable-scroll-button-lock.... enable/disable the scroll button lock\n"
 #endif
          "--set-click-method=[none|clickfinger|buttonareas] .... set the desired click method\n"
+#ifdef HAVE_LIBINPUT_1_26
+         "--set-clickfinger-map=[lrm|lmr].... set button mapping for clickfinger\n"
+#endif
          "--set-scroll-method=[none|twofinger|edge|button] ... set the desired scroll method\n"
          "--set-scroll-button=BTN_MIDDLE ... set the button to the given button code\n"
-         "--set-profile=[adaptive|flat].... set pointer acceleration profile\n"
          "--set-speed=<value>.... set pointer acceleration speed (allowed range [-1, 1]) \n"
          "--set-tap-map=[lrm|lmr] ... set button mapping for tapping\n"
+         "--set-rotation-angle=<degrees>.... set the rotation angle in degrees (0 to 360)\n"
+#ifdef HAVE_LIBINPUT_1_23
+         "--set-profile=[adaptive|flat|custom].... set pointer acceleration profile\n"
+         "If profile is custom:\n"
+         "  --set-custom-points=0.0;1.0 set the n points defining a custom acceleration function\n"
+         "  --set-custom-step=1.0 sets the distance along the x-axis between each point, starting from 0\n"
+         "  --set-custom-type=[fallback|motion|scroll] set the type of the custom acceleration function\n"
+#else
+         "--set-profile=[adaptive|flat].... set pointer acceleration profile\n"
+#endif
          "\n"
          "These options apply to all applicable devices, if a feature\n"
          "is not explicitly specified it is left at each device's default.\n"
