@@ -170,7 +170,7 @@ void set_lut(const char *def)
     char subcode;
     char padding[3];
     uint32_t lut[8];
-  } l = { TIOCL_SELLOADLUT, 0, 0, 0,
+  } l = { TIOCL_SELLOADLUT, { 0, 0, 0 }, {
     0x00000000, /* control chars     */
     0x03FFE000, /* digits and "-./"  */
     0x87FFFFFE, /* uppercase and '_' */
@@ -179,6 +179,7 @@ void set_lut(const char *def)
     0x00000000,
     0xFF7FFFFF, /* latin-1 accented letters, not multiplication sign */
     0xFF7FFFFF  /* latin-1 accented letters, not division sign */
+    }
   }; /* all of Unicode above U+00FF is considered "word" chars, even
         frames and the likes */
 
