@@ -1,5 +1,5 @@
 /* Adapted from test/shared.c from the libinput distribution */
-/* tools/shared.c 1.21.0:
+/* tools/shared.c 1.23.0:
  *
  * Copyright © 2014 Red Hat, Inc.
  * Modifications: Copyright © 2019 Bill Allombert <ballombe@debian.org>
@@ -39,22 +39,7 @@
 #include <libevdev/libevdev.h>
 
 #include "shared.h"
-
-#define streq(s1, s2) (strcmp((s1), (s2)) == 0)
-
-static inline char*
-safe_strdup(const char *str)
-{
-        char *s;
-
-        if (!str)
-                return NULL;
-
-        s = strdup(str);
-        if (!s)
-                abort();
-        return s;
-}
+#include "util-strings.h"
 
 static void
 log_handler(struct libinput *li,
@@ -84,6 +69,14 @@ tools_init_options(struct tools_options *options)
 	options->scroll_button_lock = -1;
 	options->speed = 0.0;
 	options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_NONE;
+	/* initialize accel args */
+#ifdef HAVE_LIBINPUT_1_23
+	static double points[] = {0.0, 1.0};
+	options->custom_points = points;
+	options->custom_npoints = ARRAY_LENGTH(points);
+	options->custom_type = LIBINPUT_ACCEL_TYPE_FALLBACK;
+	options->custom_step = 1.0;
+#endif
 }
 
 int
@@ -173,6 +166,22 @@ tools_parse_option(int option,
 			return 1;
 		}
 		break;
+	case OPT_CLICKFINGER_MAP:
+		if (!optarg)
+			return 1;
+
+		if (streq(optarg, "lrm")) {
+#ifdef HAVE_LIBINPUT_1_26
+			options->clickfinger_map = LIBINPUT_CONFIG_CLICKFINGER_MAP_LRM;
+#endif
+		} else if (streq(optarg, "lmr")) {
+#ifdef HAVE_LIBINPUT_1_26
+			options->clickfinger_map = LIBINPUT_CONFIG_CLICKFINGER_MAP_LMR;
+#endif
+		} else {
+			return 1;
+		}
+		break;
 	case OPT_SCROLL_METHOD:
 		if (!optarg)
 			return 1;
@@ -226,6 +235,10 @@ tools_parse_option(int option,
 			options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_ADAPTIVE;
 		else if (streq(optarg, "flat"))
 		      options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_FLAT;
+#ifdef HAVE_LIBINPUT_1_23
+		else if (streq(optarg, "custom"))
+		      options->profile = LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM;
+#endif
 		else
 		      return 1;
 		break;
@@ -247,8 +260,55 @@ tools_parse_option(int option,
 			 "%s",
 			 optarg);
 		break;
-	}
+	case OPT_CUSTOM_POINTS:
+		if (!optarg)
+			return 1;
+#ifdef HAVE_LIBINPUT_1_23
+		options->custom_points = double_array_from_string(optarg,
+								  ";",
+								  &options->custom_npoints);
+		if (!options->custom_points || options->custom_npoints < 2) {
+			fprintf(stderr,
+				"Invalid --set-custom-points\n"
+				"Please provide at least 2 points separated by a semicolon\n"
+				" e.g. --set-custom-points=\"1.0;1.5\"\n");
+			return 1;
+		}
+#endif
+		break;
+	case OPT_CUSTOM_STEP:
+		if (!optarg)
+			return 1;
+#ifdef HAVE_LIBINPUT_1_23
+		options->custom_step = strtod(optarg, NULL);
+#endif
+		break;
+	case OPT_CUSTOM_TYPE:
+		if (!optarg)
+			return 1;
+#ifdef HAVE_LIBINPUT_1_23
+		if (streq(optarg, "fallback"))
+			options->custom_type = LIBINPUT_ACCEL_TYPE_FALLBACK;
+		else if (streq(optarg, "motion"))
+			options->custom_type = LIBINPUT_ACCEL_TYPE_MOTION;
+		else if (streq(optarg, "scroll"))
+			options->custom_type = LIBINPUT_ACCEL_TYPE_SCROLL;
+		else {
+			fprintf(stderr, "Invalid --set-custom-type\n"
+			                "Valid custom types: fallback|motion|scroll\n");
+			return 1;
+		}
+#endif
+		break;
+	case OPT_ROTATION_ANGLE:
+		if (!optarg)
+			return 1;
 
+		if (!safe_atou(optarg, &options->angle)) {
+			fprintf(stderr, "Invalid --set-rotation-angle value\n");
+			return 1;
+		}
+	}
 	return 0;
 }
 
@@ -414,6 +474,12 @@ tools_device_apply_config(struct libinput_device *device,
 	if (options->click_method != (enum libinput_config_click_method)-1)
 		libinput_device_config_click_set_method(device, options->click_method);
 
+#ifdef HAVE_LIBINPUT_1_26
+	if (options->clickfinger_map != (enum libinput_config_clickfinger_button_map)-1)
+		libinput_device_config_click_set_clickfinger_button_map(device,
+									options->clickfinger_map);
+#endif
+
 	if (options->scroll_method != (enum libinput_config_scroll_method)-1)
 		libinput_device_config_scroll_set_method(device,
 							 options->scroll_method);
@@ -434,6 +500,22 @@ tools_device_apply_config(struct libinput_device *device,
 			libinput_device_config_accel_set_profile(device,
 								 options->profile);
 	}
+#ifdef HAVE_LIBINPUT_1_23
+	if (options->profile == LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM) {
+		struct libinput_config_accel *config =
+			libinput_config_accel_create(LIBINPUT_CONFIG_ACCEL_PROFILE_CUSTOM);
+		libinput_config_accel_set_points(config,
+						 options->custom_type,
+						 options->custom_step,
+						 options->custom_npoints,
+						 options->custom_points);
+		libinput_device_config_accel_apply(device, config);
+		libinput_config_accel_destroy(config);
+	}
+#endif
+
+	if (options->angle != 0)
+		libinput_device_config_rotation_set_angle(device, options->angle % 360);
 }
 
 static char*
@@ -461,7 +543,7 @@ find_device(const char *udev_tag)
 			continue;
 
 		sysname = udev_device_get_sysname(device);
-		if (strncmp("event", sysname, 5) != 0) {
+		if (!strneq("event", sysname, 5)) {
 			udev_device_unref(device);
 			continue;
 		}
